/*************************************************************************
 * Copyright (C) 2001-2004 Linus kerlund <puterman at fairlight dot st>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************/

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "callbacks.h"
#include "ui.h"

GtkWidget *about_dialog = NULL;
GtkWidget *about_button = NULL;
GtkWidget *about_label = NULL;

char *cwd = NULL;
gchar *selected_file = NULL;

extern int mixing;

void
delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit ();
}

/*
 * Well, well, well, if it isn't a little memory leak...
 */

void
delete_dialog (GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy (GTK_WIDGET (data));
}

/*
 * delete_dialog and about don't work very well...
 * Should probably block all signals to the main window until
 * the about dialog is destroyed.
 */

void
about (GtkWidget *widget, gpointer data)
{
  about_dialog = gtk_dialog_new ();

  about_button = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->action_area),
		      about_button, TRUE, TRUE, 2);
  gtk_signal_connect (GTK_OBJECT (about_button), "clicked",
		      GTK_SIGNAL_FUNC (delete_dialog), (gpointer)about_dialog);
  gtk_widget_show (about_button);

  about_label = gtk_label_new ("Vicshow GTK by Linus Åkerlund \n"
			       "\n"
			       "Based on Vicshow for MSDOS by Piotr Tracz\n"
			       "\n"
			       "(c) 2001-2004 Linus Åkerlund");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->vbox),
		      about_label, TRUE, TRUE, 5);
  gtk_widget_show (about_label);

  gtk_widget_show (about_dialog);
}
  
void
print_hello (GtkWidget *widget, gpointer data)
{
  g_print ("Hello!\n");
}

void
callback_next_pic(GtkWidget *widget, gpointer data)
{
  next_pic();
}

void
callback_prev_pic(GtkWidget *widget, gpointer data)
{
  prev_pic();
}

char *
get_base(char *path)
{
  char *end;
  char *p = strdup(path);
  
  end = strrchr(p, '/');

  *end = '\0';

  return p;
}

static void
store_filename(GtkWidget *widget, gpointer sel)
{
  selected_file = 
		(gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(sel));

  add_pic_to_list(selected_file);

  if (cwd)
    free(cwd);
  
  cwd = strdup(get_base(selected_file));

  chdir(cwd);

  show_pic();
}

static void
destroy_file_sel(GtkWidget *widget, gpointer user_data)
{
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

void
callback_open_pic(GtkWidget *widget, gpointer data)
{
  GtkWidget *file_sel = gtk_file_selection_new("Open file");

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->ok_button),
		     "clicked", GTK_SIGNAL_FUNC(store_filename),
		     (gpointer)file_sel);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->ok_button),
		     "clicked", GTK_SIGNAL_FUNC(destroy_file_sel),
		     (gpointer)file_sel);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->cancel_button),
		     "clicked", GTK_SIGNAL_FUNC(destroy_file_sel),
		     (gpointer)file_sel);
  
  gtk_widget_show(file_sel);
}

void
callback_mixing(GtkWidget *widget, gpointer data)
{
  if (mixing)
    mixing = 0;
  else
    mixing = 1;

  show_pic();
}

void
save_screenshot(GtkWidget *widget, gpointer data)
{
  FILE *f;
  gchar *filename = 
		(gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));

  f = fopen(filename, "wb");
  if (!f) {
    callback_error_dialog("Couldn't open output file!");
    return;
  }
  
  printf("Saving file %s...\n", filename);

  /* PPM header */
  fprintf(f, "P6\n320 200\n255\n");

  ui_save_image_buffer(f);
  
  fclose(f);
}

void
callback_save_screenshot(GtkWidget *widget, gpointer data)
{
  GtkWidget *file_sel_save = gtk_file_selection_new("Save screenshot");

  gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel_save),
				  "screenshot.ppm");
  
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel_save)->ok_button),
		     "clicked", GTK_SIGNAL_FUNC(save_screenshot),
		     (gpointer)file_sel_save);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel_save)->ok_button),
		     "clicked", GTK_SIGNAL_FUNC(destroy_file_sel),
		     (gpointer)file_sel_save);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel_save)->
				cancel_button),
		     "clicked", GTK_SIGNAL_FUNC(destroy_file_sel),
		     (gpointer)file_sel_save);
  
  gtk_widget_show(file_sel_save);
}

/*
 * Throws up an error dialog on the screen.
 */

void
callback_error_dialog(gchar *msg)
{
  GtkWidget *error_dialog = gtk_dialog_new ();
  GtkWidget *error_label;
  GtkWidget *error_button = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->action_area),
		      error_button, TRUE, TRUE, 2);
  gtk_signal_connect (GTK_OBJECT (error_button), "clicked",
		      GTK_SIGNAL_FUNC (delete_dialog), (gpointer)error_dialog);
  gtk_widget_show (error_button);

  error_label = gtk_label_new (msg);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->vbox),
		      error_label, TRUE, TRUE, 5);
  gtk_widget_show (error_label);

  gtk_widget_show (error_dialog);
}
