/*************************************************************************
 * Copyright (C) 2001-2004 Linus kerlund <puterman at fairlight dot st>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************/

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <ctype.h>
#include <unistd.h>
#include "ui.h"
#include "callbacks.h"
#include "vicshow.h"

#define WIDTH 640
#define HEIGHT 400

/* FIXME: Not all of these have to be global. */
int mixing = 0;
gint current_pic;
GSList *files;
guchar rgbbuf[WIDTH * HEIGHT * 3];
GtkWidget *window;
GtkWidget *label, *label2;
GtkWidget *drawing_area;
GtkActionGroup *action_group;
GtkUIManager *ui_manager;
GtkAccelGroup *accel_group;
GError *error;
static GtkActionEntry entries[] = {
	{ "FileMenu", NULL, "_File" },
	{ "PreferencesMenu", NULL, "_Preferences" },
	{ "HelpMenu", NULL, "_Help" },
	{ "Open", NULL, "_Open", "<control>O", "Open a file", 
		G_CALLBACK(callback_open_pic) },
	{ "Next", NULL, "_Next Picture", "<control>N", "Show next picture",
		G_CALLBACK(callback_next_pic) },
	{ "Prev", NULL, "_Previous Picture", "<control>P", "Show previous picture",
		G_CALLBACK(callback_prev_pic) },
	{ "Save", NULL, "_Save Screenshot", "<control>S", "", 
		G_CALLBACK(callback_save_screenshot) },
	{ "Quit", NULL, "_Quit", "<control>Q", "", G_CALLBACK(delete_event) },
	{ "About", NULL, "_About", "<control>A", "", G_CALLBACK(about) }
};

static GtkToggleActionEntry toggle_entries[] = {
	{ "ColourMixing", NULL, "Colour _Mixing", "<control>M", "", 
		G_CALLBACK(callback_mixing) }
};

static const char *ui_description =
"<ui>"
"  <menubar name='MainMenu'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Open'/>"
"      <menuitem action='Next'/>"
"      <menuitem action='Prev'/>"
"      <menuitem action='Save'/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='PreferencesMenu'>"
"      <menuitem action='ColourMixing'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

/*
 * Move on to the next picture, if there is a next picture.
 */

gint
next_pic(void)
{
  if (g_slist_length(files) == 1)
    return 1;

  if (current_pic >= g_slist_length(files)-2)
    return 1;

  current_pic++;

  show_pic();
  
  return 0;
}

/*
 * Go back to the previous picture, if there is a previous picture.
 */

gint
prev_pic(void)
{
  if (current_pic == 0)
    return 1;

  current_pic--;

  show_pic();
  
  return 0;
}

/*
 * Remove current picture from list of pictures (files).
 */

void
remove_pic_from_list(gint n)
{
  g_slist_remove(files, g_slist_nth(files, n)->data);

  if (current_pic > g_slist_length(files) - 2)
    current_pic--;

  if (current_pic < 0)
    current_pic = 0;

  if (g_slist_length(files) > 1)
    show_pic();
}

/*
 * Convert filename to UTF8, if necessary.
 */

static gchar *
convert_filename(const char *filename)
{
	gchar *ret = NULL;
	gsize bytes_read, bytes_written;
	GError *error = NULL;

	ret = g_filename_to_utf8(filename, -1, &bytes_read, &bytes_written, &error);
	
	return ret;
}

/*
 * Display picture, and modify title labels accordingly.
 */

void
show_pic(void)
{
	gchar *utf8name;
	char *ret;
  char *picture;

  picture = (char *)(g_slist_nth(files, current_pic)->data);
  if (!picture)
    return;

	utf8name = convert_filename((gchar *)g_slist_nth(files, current_pic)->data);

  gtk_label_set_text(GTK_LABEL(label), utf8name);

	ret = vicshow_main(rgbbuf, picture, mixing);

  if (!ret)
    remove_pic_from_list(current_pic);

  /* This is to get it to redraw. */
  gtk_widget_hide(drawing_area);
  gtk_widget_show(drawing_area);
  
  gtk_label_set_text(GTK_LABEL(label2), ret);
}

/*
 * Write current image to file f.
 */

void
ui_save_image_buffer(FILE *f)
{
  int i, j, k, offs = 0;
 
	for (i = 0; i < 200; i++) {
		for (j = 0; j < 320; j++) {
			for (k = 0; k < 3; k++) {
				if (fputc(rgbbuf[offs], f) == EOF) {
      		callback_error_dialog("Error writing screenshot to file!");
      		return;
				}
				offs++;
			}
			offs += 3;
		}
		offs += WIDTH * 3;
	}
}

/*
 * Returns 1 if file is a regular file (ie. a possible picture), otherwise 0.
 */

int
is_file(const char *filename)
{
  struct stat s;

  if (stat(filename, &s) != 0)
    return 0;

  if (S_ISREG(s.st_mode))
    return 1;

  return 0;
}

/*
 * Add picture filename to list of pictures (files).
 */

void
add_pic_to_list(const char *filename)
{
	gchar *utf8name = NULL;

  if (!is_file(filename))
    return;
 
	utf8name = convert_filename(filename);
 
  if (g_slist_length(files) == 1)
    files = g_slist_prepend(files, (gpointer)strdup(filename));
  else
    files = g_slist_insert(files, (gpointer)strdup(filename),
			   g_slist_length(files) - 1);

  current_pic = g_slist_length(files) - 2;
}

/*
 * Return full path of file.  The returned string is allocated, and
 * should be freed by the calling function.
 */

static char *
full_path(char *file)
{
  char *temp;
  char *path;

  if (file[0] == '/')
    return (strdup(file));

  temp = (char *)malloc(sizeof(char) * 1024);
  if (!temp) {
    fprintf(stderr, "Error allocating memory!\n");
    exit(1);
  }

  temp = getcwd(temp, 1023);
  if (!temp) {
    /* Maybe we shouldn't exit because of this, but set CWD to "/" */
    callback_error_dialog("Error getting current working directory!\n");
    exit(1);
  }

  path = (char *)malloc(sizeof(char) * (strlen(temp) + strlen(file) + 2));
  if (!path) {
    fprintf(stderr, "Error allocating memory!\n");
    exit(1);
  }

  strcpy(path, temp);
  strcat(path, "/");
  strcat(path, file);

  free(temp);

  return path;
}

/*
 * Add argv pictures to list of pictures.
 */

static int
add_pics_to_list_argv(int argc, char *argv[])
{
  int num = 0, i;

  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-')
      continue;

    num++;
    add_pic_to_list(full_path(argv[i]));
  }

  current_pic = 0;
  
  return num;
}

/*
 * Create a menu bar.
 */

static void
get_main_menu(GtkWidget  *window, GtkWidget **menubar)
{
	action_group = gtk_action_group_new("MenuActions");
	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), 
		window);
	gtk_action_group_add_toggle_actions (action_group, toggle_entries, 
		G_N_ELEMENTS (toggle_entries), window);

	ui_manager = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);

	accel_group = gtk_ui_manager_get_accel_group(ui_manager);
	gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

	error = NULL;
	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, 
		&error)) {
    g_message ("building menus failed: %s", error->message);
    g_error_free (error);
    exit (EXIT_FAILURE);
  }

	*menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
}

/*
 * Redrawing function for the drawing area.
 */

static gboolean
drawing_area_expose(GtkWidget *widget, GdkEventExpose *event,
		    gpointer *user_data)
{
  gdk_draw_rgb_image(widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
		     0, 0, WIDTH, HEIGHT, GDK_RGB_DITHER_MAX, rgbbuf,
		     WIDTH * 3);

  return TRUE;
}

/*
 * Function: main
 *   Parses arguments, creates some widgets.
 */

int
main(int argc, char **argv)
{
  int pics;
  GtkWidget *box_main;
  GtkWidget *menubar;
  
  gtk_init(&argc, &argv);
  
  /* init GTK and stuff here... */

  files = g_slist_alloc();
  if (!files) {
    fprintf(stderr, "Failed to allocate memory!\n");
    exit(1);
  }
  
  pics = add_pics_to_list_argv(argc, argv);

  /* window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "vicshow GTK");
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		      GTK_SIGNAL_FUNC(delete_event), NULL);
  gtk_container_set_border_width(GTK_CONTAINER(window), 2);

  /* boxes */
  box_main = gtk_vbox_new(FALSE, 0);
  
  /* Menus */
  get_main_menu(window, &menubar);

  gtk_box_pack_start(GTK_BOX(box_main), menubar, FALSE, FALSE, 0);

  gtk_widget_show(menubar);

  /* Drawing area */
  gdk_rgb_init();

  gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
  gtk_widget_set_default_visual(gdk_rgb_get_visual());

  drawing_area = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), WIDTH, HEIGHT);
  gtk_container_add(GTK_CONTAINER(box_main), drawing_area);
  gtk_signal_connect(GTK_OBJECT(drawing_area), "expose-event",
		     GTK_SIGNAL_FUNC(drawing_area_expose), NULL);
  gtk_widget_show(drawing_area);

  /* Label */
  label = gtk_label_new("No picture");
  gtk_container_add(GTK_CONTAINER(box_main), label);
  gtk_widget_show(label);

  gtk_label_set_text(GTK_LABEL(label),
		     (char *)(g_slist_nth(files, current_pic)->data));

  label2 = gtk_label_new("");
  gtk_container_add(GTK_CONTAINER(box_main), label2);
  gtk_widget_show(label2);

  if (pics)
    show_pic();
  
  /* Finish it off */
  gtk_widget_show(box_main);

  gtk_container_add(GTK_CONTAINER(window), box_main);

  gtk_widget_show(window);

  gtk_main();

  return 0;
}  
