/** ------------------------- **/
/**    VicShow main routine   **/
/** (c) 1996-1999 Piotr Tracz **/
/**    All Rights Reserved    **/
/**       piotr@shape.tm      **/
/** ------------------------- **/

//exclusively for olav mrkrid <olav@bogus.net> aka olav / panoramic design

//color palette by daniel kottmair aka deekay / crest

//orginaly written for Watcom C/C++ compiler but this source should work on any other compiler

//big thanks to Olav for giving me his "C64 image decoder" source-code which help a lot with Zoomatic support!


/** last updated 19.07.2000 **/
// ver. 2.2

// There is once one thing to be improved; adding support to the first three columns in FLI Interlace mode, as seems
// not all graphicians has noticed that Funapaint II from Manfred Trenz gives the same abilities as ASP's FLI-Graph.

//VicShow recognizes the following graphic file formats:

// Funpaint II (both with and without RLE compression) [320x200]
// Gunpaint [320x200]
// DrazLace v1.0 (both with and without RLE compression) [320x200]
// FLI Designer [160x200]
// Unknown FLI [160x200]
// Dolphin's FLI Editor [160x200]
// FLI Graph (both with and without RLE compression) [160x200]
// Advanced OCP Art Studio [160x200]
// Faces Painter [160x200]
// Koala Painter [160x200]
// Amica Paint [160x200]
// HiRes Bitmap [320x200]
// Art Studio [320x200]
// Topaz HiRes Editor [320x200]
// Topaz AFLI Editor [320x200]
// Zoomatic (both with and without RLE compression) [160x200]
// Paint Magic [160x200]
// Doodle [320x200]
// Coder format (bitmap at 2000, screen at 4000, colour at 4400, bg at 3f40)

//----------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
/* #include <io.h> */
/* #include <conio.h> */
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

#include <assert.h>

#include <gtk/gtk.h>
#include "ui.h"

#define WIDTH 640
#define HEIGHT 400

unsigned char vidmem[320*200]; /* = (char *)0xA0000; */
unsigned char paletter[256];
unsigned char paletteg[256];
unsigned char paletteb[256];

unsigned char funpaintII[] = {0xf0,0x3f,0x46,0x55,0x4e,0x50,0x41,0x49,0x4e,0x54,0x20,0x28,0x4d,0x54,0x29,0x20};
unsigned char gunpaint[] = {0x47,0x55,0x4e,0x50,0x41,0x49,0x4e,0x54,0x20,0x28,0x4a,0x5a,0x29};
unsigned char drazlace[] = {0x00,0x58,0x44,0x52,0x41,0x5a,0x4c,0x41,0x43,0x45,0x21,0x20,0x31,0x2e,0x30};
unsigned char fligraph[] = {0xf0,0x38,0xff,0xce,0x39,0x3f,0x7f,0x00,0x00};

unsigned char pal_r[] = {10,255,139,101,167,79,27,243,163,63,203,69,131,151,79,187};
unsigned char pal_g[] = {10,255,31,205,59,179,13,235,71,28,123,68,131,255,147,187};
unsigned char pal_b[] = {10,255,0,168,159,23,147,81,0,0,111,68,131,151,211,187};


unsigned char BMPHeader[] = {0x42,0x4D,0x36,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0x04,0x00,0x00,0x28,0x00,0x00,0x00,0x40,0x01,0x00,0x00,0xC8,0x00,0x00,0x00,0x01,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0xFA,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

unsigned char eor[] = {6,4,2,0};
unsigned char eor8[] = {7,6,5,4,3,2,1,0};
unsigned char *LoadBuffer,*TempBuffer,*screen,cea;
unsigned int inputsize, inhandle, temphandle;
unsigned char GraphicType, ColorMixing, MixingAvaliable, BackGroundColor;
unsigned char *bmpname;
FILE *bmpfile;


/*
#pragma aux SetVideoMode =\
        "int 10h",\
        parm [ax] modify[ax];
*/

/* This function isn't available on my system, so I have to define it */

static unsigned int _rotr(unsigned int value, unsigned int shift)
{
  unsigned int i, last;
  unsigned int high_bit = UINT_MAX - (UINT_MAX / 2);
  
  for (i = 0; i < shift; i++) {
    last = value & 1;
    value >>= 1;
    value |= last ? high_bit : 0;
  }

  return value;
}

// this rutine sets the color palette. If both ColorMixing and MixingAvailiable are equal to 0 then simple 16
// colors palette is created, otherwise 16x16 colors palette is created (all possible color variations)
/* Now, this is the X version, so we create a palette array instead... */
static void Set13hPalette (void)
{
  unsigned char pal_1,pal_2;
  if ((ColorMixing & MixingAvaliable) == 0)
    {
      for (pal_1=0;pal_1<16;pal_1++)
	{
	  paletter[pal_1] = pal_r[pal_1];
	  paletteg[pal_1] = pal_g[pal_1];
	  paletteb[pal_1] = pal_b[pal_1];
	  
	  /*	  outp (0x3c8, pal_1);
		  outp (0x3c9, pal_r[pal_1]/4);
		  outp (0x3c9, pal_g[pal_1]/4);
		  outp (0x3c9, pal_b[pal_1]/4); */
	}

    }
  else
    {
      for (pal_1=0;pal_1<16;pal_1++)
	{
	  for (pal_2=0;pal_2<16;pal_2++)
	    {
	      paletter[pal_1*16 + pal_2] = (pal_r[pal_1]+pal_r[pal_2])/2;
	      paletteg[pal_1*16 + pal_2] = (pal_g[pal_1]+pal_g[pal_2])/2;
	      paletteb[pal_1*16 + pal_2] = (pal_b[pal_1]+pal_b[pal_2])/2;

	      /*	      outp (0x3c8, pal_1*16+pal_2);
			      outp (0x3c9, (pal_r[pal_1]/4+pal_r[pal_2]/4)/2);
			      outp (0x3c9, (pal_g[pal_1]/4+pal_g[pal_2]/4)/2);
			      outp (0x3c9, (pal_b[pal_1]/4+pal_b[pal_2]/4)/2); */
	    }
		}
	}
}

#if 0
static void SaveBMPPalette (void)
{
  unsigned char pal_1,pal_2;
  if ((ColorMixing & MixingAvaliable) == 0)
    {
      for (pal_1=0;pal_1<16;pal_1++)
	{
	  fputc (pal_b[pal_1], bmpfile);
	  fputc (pal_g[pal_1], bmpfile);
	  fputc (pal_r[pal_1], bmpfile);
	  fputc (0, bmpfile);

	}
      for (pal_1=0;pal_1<240;pal_1++)
	{
	  fputc (0, bmpfile);
	  fputc (0, bmpfile);
	  fputc (0, bmpfile);
	  fputc (0, bmpfile);
	}

    }
  else
    {
      for (pal_1=0;pal_1<16;pal_1++)
	{
	  for (pal_2=0;pal_2<16;pal_2++)
	    {
	      fputc ((pal_b[pal_1]+pal_b[pal_2])/2, bmpfile);
	      fputc ((pal_g[pal_1]+pal_g[pal_2])/2, bmpfile);
	      fputc ((pal_r[pal_1]+pal_r[pal_2])/2, bmpfile);
	      fputc (0, bmpfile);
	    }
	}
    }
}
#endif

// this rutines makes a screen dump into 320x200x256 Windows BMP file. No color optimization is made in order to make
// possible pallete changing in image processing programs         
#if 0
static void SaveBMPImage (void)
{
  int temp;
  unsigned int counter_1,counter_2, pointer;
  temp = 0;
  counter_1 =0;

  for (temp=0;temp<8;temp++)
    {
      if (bmpname[temp]=='.') break;
    }
  temp++;
  bmpname[temp]='b';
  temp++;
  bmpname[temp]='m';
  temp++;
  bmpname[temp]='p';
  temp++;
  bmpname[temp]='\0';

  bmpfile = fopen(bmpname, "wb");
  fwrite(BMPHeader, sizeof(BMPHeader),1, bmpfile);

  SaveBMPPalette();

  pointer=63680;

  for (counter_2=0;counter_2<200;counter_2++)
    {
      for (counter_1=0;counter_1<320;counter_1++)
	{
	  /* FIXME: change
	  fputc (vidmem[pointer+counter_1], bmpfile);
	  */
	}
      pointer = pointer-320;
    }


  fclose (bmpfile);
}
#endif

static unsigned char CheckZoomaticFile (void)
{
  unsigned char byte;
  unsigned int repeat,x,position_b;
  int position_a;
  position_a = inputsize-2;
  position_b = 0;

 depack: byte = LoadBuffer[position_a];
  if (position_a < 0) goto depack_end;
  if (byte != LoadBuffer[inputsize-1])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a--;
  repeat = LoadBuffer[position_a];
  if (repeat == 0) repeat = 256;
  position_a--;
 write_bytes: byte = LoadBuffer[position_a];
  for (x=0;x<repeat;x++)
    {
      position_b++;
    }
  position_a--;
  goto depack;
        
 depack_end: if (position_b == 10003) return (0);
 else return (1);
        
}        

/**  ---------------------- **/
/** checking graphic format **/
/**                         **/
/** 0x00 - unknown          **/
/** 0x01 - funpaintII (p)   **/
/** 0x02 - funpaintII (u)   **/
/** 0x03 - gunpaint         **/
/** 0x04 - flidesigner      **/
/** 0x05 - Adv. Art Studio  **/
/** 0x06 - DrazLace v1.0 (u)**/
/** 0x07 - DrazLace v1.0 (p)**/
/** 0x04 - unknown FLI 3c00 **/
/** 0x08 - Dolphin's FLI    **/
/** 0x09 - Faces' Painter   **/
/** 0x0a - Koala Painter    **/
/** 0x0b - Amica Paint      **/
/** 0x0c - Hires Bitmap     **/
/** 0x0d - Fli Graph 2.2 (u)**/
/** 0x0e - Fli Graph 2.2 (p)**/
/** 0x0f - Art Studio 1.0   **/
/** 0x10 - Topaz Hires v1.0 **/
/** 0x11 - Topaz AFLI v2.0  **/
/** 0x12 - Paint Magic      **/
/** 0x13 - Zoomatic (p)     **/
/** 0x14 - Zoomatic (u)     **/
/** 0x15 - Doodle           **/
/**                         **/
/** 0x16 - Coder format     **/
/** ----------------------- **/
// this routines checks the file format and returns value assigned to each C64 format (see above)
// there is a very little chance of wrong file format recogization but it may only happen if user made a trojan
// file with premeditation. However there is no possiblity of making any runtime error when any routines will try
// to process any file which truly is none of supported formats.


static unsigned char CheckGraphicFileFormat ()
{
  /** checking FunpaintII format **/
  unsigned char result;
  result = 0;
  if (memcmp(LoadBuffer,funpaintII,sizeof(funpaintII)) == 0)
    {
      if (LoadBuffer[0x10] != 0)
	{
	  result = 1;
	  MixingAvaliable = 1;
	}
      else
	{
	  result = 2;
	  MixingAvaliable = 1;
	}
    }
  /** checking Gunpaint **/

  if (memcmp(LoadBuffer+0x3ea,gunpaint,sizeof(gunpaint)) == 0)
    {
      result = 3;
      MixingAvaliable = 1;
    }
  /** checking FLIDesigner **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x3c && inputsize == 17409)
    {
      result = 4;
      MixingAvaliable = 0;
    }
  /** checking Adv. Art Studio **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x20 && inputsize == 10018)
    {
      result = 5;
      MixingAvaliable = 0;
    }
  /** checking DrazLace v1.0 **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x58 && inputsize == 18242)
    {
      result = 6;
      MixingAvaliable = 1;
    }
  /** checking DrazLace v1.0 Packed **/
  if (memcmp(LoadBuffer,drazlace,sizeof(drazlace)) == 0)
    {
      result = 7;
      MixingAvaliable = 1;
    }

  /** checking unknown FLI **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x3c && inputsize == 17410)
    {
      result = 4;
      MixingAvaliable = 0;
    }
  /** checking Dolphin's FLI **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x3b && inputsize == 17665)
    {
      result = 8;
      MixingAvaliable = 0;
    }
  /** checking FacesPainter **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x40 && inputsize == 10004)
    {
      result = 9;
      MixingAvaliable = 0;
    }
  /** checking KoalaPaint **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x60 && inputsize == 10003)
    {
      result = 10;
      MixingAvaliable = 0;
    }
  /** checking Amica Paint **/
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x40 && LoadBuffer[2] == LoadBuffer[inputsize-2] && LoadBuffer[inputsize-1] == 0 && inputsize < 12000)
    {
      result = 11;
      MixingAvaliable = 0;
    }    
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x20 && inputsize == 8002)
    {
      result = 12;
      MixingAvaliable = 0;
    }
  /** checking FLI Graph 2.2 unpacked **/     
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x3b && inputsize == 17474)
    {
      result = 13;
      MixingAvaliable = 0;
    }
  /** checking FLI Graph 2.2 packed **/     
  if (LoadBuffer[0] == 0xf0 && LoadBuffer[1] == 0x38 && LoadBuffer[5] == 0x3f && LoadBuffer[6] == 0x7f && LoadBuffer[7] == 0x00)
    {
      result = 14;
      MixingAvaliable = 0;
    }

  /** checking Art Studio v1.0 **/     
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x20 && inputsize == 9009)
    {
      result = 15;
      MixingAvaliable = 0;
    }
  /** checking Topaz HiRes Editor v1.0 **/     
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x5c && inputsize == 9217)
    {
      result = 16;
      MixingAvaliable = 0;
    }
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x40 && inputsize == 16385)
    {
      result = 17;
      MixingAvaliable = 0;
    }
  if (LoadBuffer[0] == 0x8E && LoadBuffer[1] == 0x3F && inputsize == 9332)
    {
      result = 18;
      MixingAvaliable = 0;
    }
  if (CheckZoomaticFile() == 0)
    {
      result = 19;
      MixingAvaliable = 0;
    }
  /** checking Zoomatic (u) **/                  
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x60 && inputsize == 10003)
    {
      result = 20;
      MixingAvaliable = 0;                       
    }
  /** checking Doodle **/     
  if (LoadBuffer[0] == 0x00 && LoadBuffer[1] == 0x5c && inputsize == 9218)
    {
      result = 21;
      MixingAvaliable = 0;
    }
	/** Checking Coder format **/
	if (inputsize == 10218)
		{
			result = 0x16;
			MixingAvaliable = 0;
		}
		
  return (result);
}

/** allocete all nescesary buffers **/
static void AllocateBuffers()
{ /* Was: ((...) == NULL) */
  if(!((LoadBuffer=malloc(0xf000))&&(screen=malloc(0xf000))&&(TempBuffer=malloc(0xf000))))
    {
      printf ("error allocating memory\n");
      exit(1);
    }
  bzero (screen,0xf000);
}

char *TellGraphicFileType (void)
{
  /*  printf ("\n you have been watching ");   */
  switch (GraphicType)
    {
    case 1:
      return ("compressed Funpaint II");
      break;
    case 2:
      return ("Funpaint II");
      break;
    case 3:
      return ("Gunpaint");                
      break;
    case 4:
      return ("FliDesigner");
      break;
    case 5:
      return ("Advanced Art Studio");
      break;
    case 6:
      return ("DrazLace");
      break;
    case 7:
      return ("compressed DrazLace");
      break;
    case 8:
      return ("Dolphin's FLI");
      break;
    case 9:
      return ("Faces Painter");
      break;
    case 10:
      return ("Koala Paint");               
      break;
    case 11:
      return ("Amica Paint");               
      break;
    case 12:
      return ("HiRes Bitmap");               
      break;
    case 13:
      return ("FLI Graph");               
      break;
    case 14:
      return ("compressed FLI Graph");               
      break;
    case 15:
      return ("Art Studio");               
      break;
    case 16:
      return ("Topaz HiRes Editor");               
      break;
    case 17:
      return ("Topaz A-FLI Editor");               
      break;
    case 18:
      return ("Paint Magic");               
      break;
    case 19:
      return ("compressed Zoomatic");               
      break;
    case 20:
      return ("uncompressed Zoomatic");               
      break;
    case 21:
      return ("Doodle");               
      break;
		case 22:
			return ("Coder format");
			break;
    }
  /*  printf (" image file\n");           */
  return NULL;
}

/** get pixel's color number from loaded Gunpaint file **/
static unsigned char GetGunpaintPixel (unsigned int x, unsigned int y, unsigned char page)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  if (page == 0)
    {
      pixels = LoadBuffer[offset + 0x2002];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x2 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x2 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x4002]&0x0f;
	  break;

	}
    }
  else
    {
      pixels = LoadBuffer[offset + 0x6402];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x4402 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x4402 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x4002]&0x0f;
	  break;

	}

    }
  return (color);
}

/** get pixel's color number from loaded FunpaintII file **/
static unsigned char GetFunpaintIIPixel (unsigned int x, unsigned int y, unsigned char page)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  if (page == 0)
    {
      pixels = LoadBuffer[offset + 0x2012];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x12 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x12 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x4012]&0x0f;
	  break;

	}
    }
  else
    {
      pixels = LoadBuffer[offset + 0x63fa];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x43fa + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x43fa + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x4012]&0x0f;
	  break;

	}

    }
  return (color);
}

static unsigned char GetDrazlacePixel (unsigned int x, unsigned int y, unsigned char page)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  if (page == 0)
    {
      pixels = LoadBuffer[offset + 0x0802];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x0402 + 40*(y/8)+x/4],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x0402 + 40*(y/8)+x/4] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x02]&0x0f;
	  break;

	}
    }
  else
    {
      pixels = LoadBuffer[offset + 0x2802];
      pixels = _rotr(pixels, eor[x&0x3]);
      switch (pixels&0x3)
	{
	case 0x00:
	  color = BackGroundColor;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x0402 + 40*(y/8)+x/4],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x0402 + 40*(y/8)+x/4] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[40*(y/8)+x/4+0x02]&0x0f;
	  break;

	}

    }
  return (color);
}

/** get pixel's color number from loaded Flidesigner file **/
static unsigned char GetFliDesignerPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2402];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x402 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x402 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[40*(y/8)+x/4+2]&0x0f;
      break;
    }
  return (color);
}

/** get pixel's color number from loaded FliGraph file **/
static unsigned char GetFliGraphPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2502];
  pixels = _rotr(pixels, eor[x&0x3]);
  if (x>11)
    { 
      switch (pixels&0x3)
	{
	case 0x00:
	  color = LoadBuffer[0x2+y]&0xf;
	  break;
	case 0x01:
	  color =  _rotr(LoadBuffer[0x502 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
	  break;
	case 0x02:
	  color = LoadBuffer[0x502 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
	  break;
	case 0x03:
	  color = LoadBuffer[0x100+40*(y/8)+x/4+2]&0x0f;
	  break;                                           
	}
      return (color);      
    }                               
  else
    {
      switch (pixels&0x3)
	{
	case 0x00:
	  color = LoadBuffer[0x2+y]&0xf; //ok
	  break;
	case 0x01:
	  color = LoadBuffer [0x502]&0xf; //ok
	  break;
	case 0x02:
	  color = LoadBuffer[0x502]&0xf;
	  break;
	case 0x03:
	  color = _rotr(LoadBuffer[0x2+y],4)&0xf; //ok
	  break;
	}
      return (color);      
                            
    }             

}

/** get pixel's color number from loaded Dolphin's FLI file **/
static unsigned char GetDolphinFLIPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2502];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x502 + 40*(y/8)+x/4+(y&0x7)*0x400],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x502 + 40*(y/8)+x/4+(y&0x7)*0x400] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[40*(y/8)+x/4+0x102]&0x0f;
      break;
    }
  return (color);
}

static void UnpackFunpaintIIFile (void)
{
  unsigned char byte, repeat, x;
  unsigned int position_a, position_b;
  position_a = 0;
  position_b = 0;

 depack: byte = LoadBuffer[0x12+position_a];
  if (byte != LoadBuffer[0x11])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a++;
  repeat = LoadBuffer[0x12+position_a];
  if (repeat == 0) goto depack_end;
  position_a++;
 write_bytes: byte = LoadBuffer[0x12 + position_a];
  for (x=0;x<repeat;x++)
    {
      TempBuffer[0x12 + position_b] = byte;
      if (position_b > 0xffe0) goto depack_end;
      position_b++;
    }
  position_a++;
  goto depack;

 depack_end: bzero (LoadBuffer,0xf000);
  /* Was: _fmemcpy() */
  memcpy(LoadBuffer, TempBuffer, 0xf000);
}

static void UnpackDrazlaceFile (void)
{
  unsigned char byte, repeat, x;
  unsigned int position_a, position_b;
  position_a = 0;
  position_b = 0;

 depack: byte = LoadBuffer[0x10+position_a];
  if (byte != LoadBuffer[0x0f])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a++;
  repeat = LoadBuffer[0x10+position_a];
  if (repeat == 0) goto depack_end;
  position_a++;
 write_bytes: byte = LoadBuffer[0x10 + position_a];
  for (x=0;x<repeat;x++)
    {
      TempBuffer[0x02 + position_b] = byte;
      if (position_b > 0x6000) goto depack_end;
      position_b++;
    }
  position_a++;
  goto depack;

 depack_end: bzero (LoadBuffer,0xf000);
  /* Was: _fmemcpy() */
  memcpy(LoadBuffer, TempBuffer, 0xf000);
}

static void UnpackAmicaPaintFile (void)
{
  unsigned char byte, repeat, x;
  unsigned int position_a, position_b;
  position_a = 0;
  position_b = 0;

 depack: byte = LoadBuffer[0x2 + position_a];
  if (byte != LoadBuffer[0x2])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a++;
  repeat = LoadBuffer[0x2+position_a];
  if (repeat == 0) goto depack_end;
  position_a++;
 write_bytes: byte = LoadBuffer[0x2 + position_a];
  for (x=0;x<repeat;x++)
    {
      TempBuffer[0x2 + position_b] = byte;
      if (position_b > 0x6000) goto depack_end;
      position_b++;
    }
  position_a++;
  goto depack;

 depack_end: bzero (LoadBuffer, 0xf000);
  /* Was: _fmemcpy() */
  memcpy (LoadBuffer, TempBuffer, 0xf000);
}

static void UnpackFLIGraphFile (void)
{
  unsigned char byte;
  unsigned int repeat,x,position_a, position_b;
  position_a = inputsize;
  position_b = 17474 - 0x2;

 depack: byte = LoadBuffer[position_a];
  if (position_b == 0x0) goto depack_end;
  if (byte != LoadBuffer[0x2])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a--;
  repeat = LoadBuffer[position_a];
  if (repeat == 0) repeat = 256;
  position_a--;
 write_bytes: byte = LoadBuffer[position_a];
  for (x=0;x<repeat;x++)
    {
      TempBuffer[0x2 + position_b] = byte;
      if (position_b == 0x0) goto depack_end;
      position_b--;
    }
  position_a--;
  goto depack;
        
 depack_end: bzero (LoadBuffer, 0xf000);
  /* Was: _fmemcpy() */
  memcpy (LoadBuffer, TempBuffer, 0xf000);       
}

static void UnpackZoomaticFile (void)
{
  unsigned char byte;
  unsigned int repeat,x,position_b;
  int position_a;
  position_a = inputsize-2;
  position_b = 10002;

 depack: byte = LoadBuffer[position_a];
  if (position_a < 0x0) goto depack_end;
  if (byte != LoadBuffer[inputsize-1])
    {
      repeat = 1;
      goto write_bytes;
    }
  else
    position_a--;
  repeat = LoadBuffer[position_a];
  if (repeat == 0) repeat = 256;
  position_a--;
 write_bytes: byte = LoadBuffer[position_a];
  for (x=0;x<repeat;x++)
    {
      TempBuffer[position_b] = byte;
      if (position_b == 0x0) goto depack_end;
      position_b--;
    }
  position_a--;
  goto depack;
        
 depack_end: bzero (LoadBuffer, 0xf000);
  /* Was: _fmemcpy() */
  memcpy (LoadBuffer, TempBuffer, 0xf000);
    
}        

/** get pixel's color number from loaded Adv.Art Studio file **/
static unsigned char GetAdvArtStudioPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x1f42 + 40*(y/8)+x/4],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x1f42 + 40*(y/8)+x/4] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[0x233a + 40*(y/8)+x/4]&0x0f;
      break;
    }
  return (color);
}
/** get pixel's color number from loaded FacesPainter file **/
static unsigned char GetFacesPainterPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x1f42 + 40*(y/8)+x/4],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x1f42 + 40*(y/8)+x/4] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[0x232a + 40*(y/8)+x/4]&0x0f;
      break;
    }
  return (color);
}
/** get pixel's color number from loaded Koala Paint file **/
static unsigned char GetKoalaPaintPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x1f42 + 40*(y/8)+x/4],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x1f42 + 40*(y/8)+x/4] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[0x232a + 40*(y/8)+x/4]&0x0f;
      break;
    }
  return (color);
}

/** get pixel's color number from loaded HiresBitmap file **/
static unsigned char GetHiresBitmapPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/8)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor8[x&0x7]);
  switch (pixels&0x1)
    {
    case 0x0:
      color = BackGroundColor;
      break;
    case 0x1:
      color = 0xf;
      break;
    }
  return (color);
}
/** get pixel's color number from loaded ArtStudio v1.0  file **/
static unsigned char GetArtStudioPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/8)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor8[x&0x7]);
  switch (pixels&0x1)
    {
    case 0x1:
      color =  _rotr(LoadBuffer[0x1f42 + 40*(y/8)+x/8],4) &0x0f;
      break;
    case 0x0:
      color = LoadBuffer[0x1f42 + 40*(y/8)+x/8] & 0x0f;                       
      break;
    }
  return (color);
}
/** get pixel's color number from loaded Topaz HiRes Editor v1.0  file **/
static unsigned char GetTopazHiresPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/8)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2 + 0x400];
  pixels = _rotr(pixels, eor8[x&0x7]);
  switch (pixels&0x1)
    {
    case 0x1:
      color =  _rotr(LoadBuffer[0x2 + 40*(y/8)+x/8],4) &0x0f;
      break;
    case 0x0:
      color = LoadBuffer[0x2 + 40*(y/8)+x/8] & 0x0f;                       
      break;
    }
  return (color);
}
/** get pixel's color number from loaded Topaz AFLI Editor v2.0  file **/
static unsigned char GetTopazAFLIPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/8)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2 + 0x2000];
  pixels = _rotr(pixels, eor8[x&0x7]);
  switch (pixels&0x1)
    {
    case 0x1:
      color =  _rotr(LoadBuffer[0x2 + 40*(y/8)+x/8+(y&0x7)*0x400],4) &0x0f;
      break;
    case 0x0:
      color = LoadBuffer[0x2 + 40*(y/8)+x/8+(y&0x7)*0x400] & 0x0f;                       
      break;
    }
  return (color);
}

/** get pixel's color number from loaded Paint Magic file **/
static unsigned char GetPaintMagicPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x74];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x2074 + 40*(y/8)+x/4],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x2074 + 40*(y/8)+x/4] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[0x1fb7]&0x0f;
      break;
    }
  return (color);
}
/** get pixel's color number from loaded Adv.Art Studio file **/
static unsigned char GetCoderFormatPixel (unsigned int x, unsigned int y)
{
  unsigned int offset;
  unsigned char pixels, color;
  /** offset **/
  offset = 320*(y/8)+8*(x/4)+(y&0x7);
  pixels = LoadBuffer[offset + 0x2];
  pixels = _rotr(pixels, eor[x&0x3]);
  switch (pixels&0x3)
    {
    case 0x00:
      color = BackGroundColor;
      break;
    case 0x01:
      color =  _rotr(LoadBuffer[0x2002 + 40*(y/8)+x/4],4) &0x0f;
      break;
    case 0x02:
      color = LoadBuffer[0x2002 + 40*(y/8)+x/4] & 0x0f;
      break;
    case 0x03:
      color = LoadBuffer[0x2402 + 40*(y/8)+x/4]&0x0f;
      break;
    }
  return (color);
}

// the next routines support original Commodore 64 graphics files. They works only on memory buffers and are faultless
// (I do hope so, I'm only not sure depack routines (mainly Zoomatic). Everyting is directly put into VideoMemory but
// if VIDMEM pointer is changed then any other buffer/memory area may be used. However the best solution is to add
// unsigned char *file_buffer , unsigned char *dest_buffer to  each function parameters. The destination is a linear pixel by pixel
// format (320x200xbyte).
//  

void ShowFLIDesigner (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetFliDesignerPixel (x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}
void ShowFLIGraph (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetFliGraphPixel (x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}

void ShowDolphinFLI (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetDolphinFLIPixel (x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}

void ShowADVArtStudio (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetAdvArtStudioPixel (x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}
void ShowFacesPainter (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetFacesPainterPixel(x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}
void ShowKoalaPaint (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetKoalaPaintPixel(x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}
void ShowPaintMagic (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetPaintMagicPixel(x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
    }
}

void ShowHiresBitmap (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<320; x++)
	{
	  pixel = GetHiresBitmapPixel(x,y);
	  vidmem[offs++] = pixel;
	}
    }
}
void ShowArtStudio (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<320; x++)
	{
	  pixel = GetArtStudioPixel(x,y);
	  vidmem[offs++] = pixel;

	}
    }
}
void ShowTopazHiRes (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<320; x++)
	{
	  pixel = GetTopazHiresPixel(x,y);
	  vidmem[offs++] = pixel;

	}
    }
}
void ShowTopazAFLI (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<320; x++)
	{
	  pixel = GetTopazAFLIPixel(x,y);
	  vidmem[offs++] = pixel;

	}
    }
}

void ShowFunpaintII (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  if (ColorMixing == 0)
    {
      for (y=0; y<200; y++)
	{
	  for (x=0; x<160; x++)
	    {
	      pixel = GetFunpaintIIPixel (x,y,0);
	      vidmem[offs++] = pixel;

	      pixel = GetFunpaintIIPixel (x,y,1);
	      vidmem[offs++] = pixel;
	    }
	}
    }
  else
    {
      for (y=0; y<200; y++)
	{
	  pixel = GetFunpaintIIPixel (0,y,0);
	  vidmem[offs++] = pixel;
	  pixel = GetFunpaintIIPixel (0,y,0)*16+GetFunpaintIIPixel(0,y,1);
	  vidmem[offs++] = pixel;

	  for (x=1; x<159; x++)
	    {
	      pixel = GetFunpaintIIPixel (x,y,0)*16+GetFunpaintIIPixel(x-1,y,1);
	      vidmem [offs++] = pixel;

	      pixel = GetFunpaintIIPixel (x,y,0)*16+GetFunpaintIIPixel(x,y,1);
	      vidmem [offs++] = pixel;

	    }
	  pixel = GetFunpaintIIPixel (159,y,0)*16+GetFunpaintIIPixel(159,y,1);
	  vidmem[offs++] = pixel;
	  pixel = GetFunpaintIIPixel (159,y,1);
	  vidmem[offs++] = pixel;
	}
    }

}
void ShowGunpaint (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  if (ColorMixing == 0)
    {
      for (y=0; y<200; y++)
	{
	  for (x=0; x<160; x++)
	    {
	      pixel = GetGunpaintPixel (x,y,0);
	      vidmem[offs++] = pixel;
	      pixel = GetGunpaintPixel (x,y,1);
	      vidmem[offs++] = pixel;
	    }
	}
    }
  else
    {
      for (y=0; y<200; y++)
	{
	  pixel = GetGunpaintPixel (0,y,0);
	  vidmem[offs++] = pixel;
	  pixel = GetGunpaintPixel (0,y,0)*16+GetGunpaintPixel(0,y,1);
	  vidmem[offs++] = pixel;

	  for (x=1; x<159; x++)
	    {
	      pixel = GetGunpaintPixel (x,y,0)*16+GetGunpaintPixel(x-1,y,1);
	      vidmem [offs++] = pixel;

	      pixel = GetGunpaintPixel (x,y,0)*16+GetGunpaintPixel(x,y,1);
	      vidmem [offs++] = pixel;

	    }
	  pixel = GetGunpaintPixel (159,y,0)*16+GetGunpaintPixel(159,y,1);
	  vidmem[offs++] = pixel;
	  pixel = GetGunpaintPixel (159,y,1);
	  vidmem[offs++] = pixel;

	}
    }


}

void ShowDrazlace (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  if (ColorMixing == 0)
    {
      for (y=0; y<200; y++)
	{
	  for (x=0; x<160; x++)
	    {
	      pixel = GetDrazlacePixel (x,y,0);
	      vidmem[offs++] = pixel;
	      pixel = GetDrazlacePixel (x,y,1);
	      vidmem[offs++] = pixel;
	    }
	}
    }
  else
    {
      for (y=0; y<200; y++)
	{
	  pixel = GetDrazlacePixel (0,y,0);
	  vidmem[offs++] = pixel;
	  pixel = GetDrazlacePixel (0,y,0)*16+GetDrazlacePixel(0,y,1);
	  vidmem[offs++] = pixel;

	  for (x=1; x<159; x++)
	    {
	      pixel = GetDrazlacePixel (x,y,0)*16+GetDrazlacePixel(x-1,y,1);
	      vidmem [offs++] = pixel;

	      pixel = GetDrazlacePixel (x,y,0)*16+GetDrazlacePixel(x,y,1);
	      vidmem [offs++] = pixel;

	    }
	  pixel = GetDrazlacePixel (159,y,0)*16+GetDrazlacePixel(159,y,1);
	  vidmem[offs++] = pixel;
	  pixel = GetDrazlacePixel (159,y,1);
	  vidmem[offs++] = pixel;

	}
    }
}

void ShowCoderFormat (void)
{
  unsigned int x,y,offs;
  unsigned char pixel;
  offs = 0;
  for (y=0; y<200; y++)
    {
      for (x=0; x<160; x++)
	{
	  pixel = GetCoderFormatPixel (x,y);
	  vidmem[offs++] = pixel;
	  vidmem[offs++] = pixel;
	}
 }
}

/** ----------------------------------------------- **/
/** graphic routines supporting C64 graphic formats **/
/** ----------------------------------------------- **/

static void ShowGraphicFile (void)
{
  switch (GraphicType)
    {
    case 1:
      UnpackFunpaintIIFile();
      ShowFunpaintII();
      break;
    case 2:
      ShowFunpaintII();
      break;
    case 3:
      ShowGunpaint();
      break;
    case 4:
      ShowFLIDesigner();
      break;
    case 5:
      ShowADVArtStudio();
      break;
    case 6:
      ShowDrazlace();
      break;
    case 7:
      ShowDrazlace();
      break;
    case 8:
      ShowDolphinFLI();
      break;
    case 9:
      ShowFacesPainter();
      break;
    case 10:
      ShowKoalaPaint();
      break;
    case 11:
      ShowKoalaPaint();
      break;
    case 12:
      ShowHiresBitmap();
      break;
    case 13:
      ShowFLIGraph();
      break;
    case 14:
      ShowFLIGraph();
      break;
    case 15:
      ShowArtStudio();
      break;
    case 16:
      ShowTopazHiRes();
      break;
    case 17:
      ShowTopazAFLI();
      break;
    case 18:
      ShowPaintMagic();
      break;
    case 19:
      ShowKoalaPaint();
      break;
    case 20:
      ShowKoalaPaint();
      break;
    case 21:
      ShowTopazHiRes();
      break;
		case 22:
			ShowCoderFormat();
			break;
    }
}

// this routine tries to get background color just before real image processing, it also depacks compressed files!
        
static void GetBackGroundColor (void)
{
  switch (GraphicType)
    {
    case 1:
      BackGroundColor = 0;
      break;
    case 2:
      BackGroundColor = 0;
      break;
    case 3:
      BackGroundColor = 0;
      break;
    case 4:
      BackGroundColor = 0;                         
      break;
    case 5:
      BackGroundColor = LoadBuffer[0x232b]&0xf;
      break;
    case 6:
      BackGroundColor = LoadBuffer[0x2741]&0xf;
      break;
    case 7:
      UnpackDrazlaceFile();
      BackGroundColor = LoadBuffer[0x2741]&0xf;
      break;
    case 8:
      BackGroundColor = 0;
      break;
    case 9:
      BackGroundColor = LoadBuffer[0x2712]&0xf;
      break;
    case 10:
      BackGroundColor = LoadBuffer[0x2712]&0xf;
      break;
    case 11:
      UnpackAmicaPaintFile();
      BackGroundColor = LoadBuffer[0x2712]&0xf;
      break;
    case 12:
      BackGroundColor = 0;
      break;
    case 13:
      BackGroundColor = 0;
      break;
    case 14:
      UnpackFLIGraphFile();                 
      BackGroundColor = 0;
      break;
    case 15:
      BackGroundColor = 0;
      break;
    case 16:
      BackGroundColor = 0;
      break;
    case 17:
      BackGroundColor = 0;
      break;
    case 18:
      BackGroundColor = LoadBuffer[0x1fb4]&0xf;
      break;
    case 19:
      UnpackZoomaticFile ();
      BackGroundColor = LoadBuffer[0x2712]&0xf;                         
      break;                         
    case 20:
      BackGroundColor = LoadBuffer[0x2712]&0xf;
      break;
    case 21:
      BackGroundColor = 0;
      break;
		case 22:
			BackGroundColor = LoadBuffer[0x1f42]&0xf;
			break;
    }
}

/** SUB-ROUTINES USED MY MAIN PROGRAM **/
// This routine checks the user's act and changes background color, mixing-mode, etc.
#if 0
static void CheckUserAct (void)
{
  unsigned char pressed;

  /* FIXME: was getch(), which I guess doesn't have to wait for ENTER. */
 checkey: pressed = getchar();
  if (pressed == 0)
    {
      /* FIXME: was getch(), see above. */
      if (getchar() == 134)
	{
	  SaveBMPImage();
	  return;
	}
    }
                    
  if (pressed == '+')
    {
      BackGroundColor = (BackGroundColor+1) & 0xf;
      ShowGraphicFile ();
      goto checkey;
    }
  if (pressed == '-')
    {
      BackGroundColor = (BackGroundColor-1) & 0xf;
      ShowGraphicFile ();
      goto checkey;
    }
  if (pressed == 'm')
    {
      ColorMixing = (ColorMixing + 1) & 0x1;
      Set13hPalette ();
      ShowGraphicFile ();
      goto checkey;
    }          
  return;
                         
}   
#endif

static void
vidmem2rgbbuf(char *rgbbuf)
{
  int i, j;

	for (i = 0; i < 400; i+=2) {
		for (j = 0; j < 320; j++) {
			rgbbuf[i*WIDTH*3 + j*2*3] = rgbbuf[i*WIDTH*3 + j*2*3 + 3] =
			rgbbuf[(i+1)*WIDTH*3 + j*2*3] = rgbbuf[(i+1)*WIDTH*3 + j*2*3 + 3] =
				paletter[vidmem[(i/2)*320 + j]];
			rgbbuf[i*WIDTH*3 + j*2*3 +1] = rgbbuf[i*WIDTH*3 + j*2*3 + 4] =
			rgbbuf[(i+1)*WIDTH*3 + j*2*3 + 1] = rgbbuf[(i+1)*WIDTH*3 + j*2*3 + 4] =
				paletteg[vidmem[(i/2)*320 + j]];
			rgbbuf[i*WIDTH*3 + j*2*3 + 2] = rgbbuf[i*WIDTH*3 + j*2*3 + 5] =
			rgbbuf[(i+1)*WIDTH*3 + j*2*3 + 2] = rgbbuf[(i+1)*WIDTH*3 + j*2*3 + 5] =
				paletteb[vidmem[(i/2)*320 + j]];
		}
	}
}

char *vicshow_main(char *rgbbuf, char *filename, int mixing)
{
  int i, ret;
  FILE *f;

  assert(filename);
  
  ColorMixing = mixing;
  
  /*
    if (vga_init() != 0) {
    fprintf(stderr, "Couldn't init svgalib, exiting.\n");
    return 1;
    }
  */

  /*
  printf ("-=VicShow v2.2 - Commodore 64 graphic files viewer=-\n");
  printf ("(c) 1996-2000 Piotr Tracz, All Rights Reserved!\n");
  */
  
  /*
    if (vga_hasmode(G320x200x256) == 0) {
    fprintf(stderr, "Sorry, videomode G320x200x256 not supported, exiting.\n");
    return 1;
    }
  */
  
  AllocateBuffers();

  bmpname = strdup(filename);

  /* SetVideoMode (0x13);  */ //set 13h video mode (320x200x256)
  /*
    vga_setmode(G320x200x256);
  
    vidmem = vga_getgraphmem();
  */

  /*
  if((inhandle = open(bmpname, O_RDONLY) < 0)) {
    printf("\n.file not found\n");
    return 1;
  }
  */

  if (!(f =  fopen(filename, "r"))) {
    printf("File not found\n");
    return NULL;
  }
  
  /** check file size and load it**/
  /*  lseek(inhandle, 0L, SEEK_END);
      inputsize = lseek(inhandle, 0, SEEK_CUR);
      lseek(inhandle,0L,SEEK_SET);
  
      printf("inputsize: %d\n", inputsize);
      
      if (inputsize < 0xffff) {
      read(inhandle, LoadBuffer, inputsize);
      }*/
  
  for (i = 0; i < 0xffff; i++) {
    ret = fgetc(f);
    if (ret == EOF)
      break;

    LoadBuffer[i] = ret;
  }

  inputsize = i;

  fclose(f);
  /*
  close (inhandle);
  */
  if ((GraphicType = CheckGraphicFileFormat()) == 0)
    {
      printf ("\n.unsupported graphic format\n");
      return NULL;
    }
  /*  
  bzero (screen,0xf000);
  */
  
  GetBackGroundColor ();

  Set13hPalette ();

  ShowGraphicFile ();

  vidmem2rgbbuf(rgbbuf);

  /*  
  CheckUserAct ();        
  */
  /* SetVideoMode (0x03); */
  /*  vga_setmode(TEXT); */
  /*  TellGraphicFileType (); */

  free(bmpname);
  
  return TellGraphicFileType();
}
